#include <iostream>
#include <fstream>

// Note that this lines should be:
// #include <curlpp/curlpp.hpp
// #include <curlpp/ftp_easy.hpp>
#include <curlpp/curlpp.hpp>
#include <curlpp/ftp_easy.hpp>

// Note that streampos.h will not be part
// of the installed curlpp package. See 
// streampos.h for more details.
// You can take a look to ICE_STREAMPOS
// and ICE_STD_STREAMPOS macros in 
// acinclude.m4
#include "streampos.h"


/*
 * This example shows an FTP upload, with a rename of the file just after
 * a successful upload.
 *
 * Example based on libcurl's examples located in "doc/examples" source code.
 */

#define LOCAL_FILE      "example13.txt"
#define UPLOAD_FILE_AS  "while-uploading.txt"
#define REMOTE_URL      "ftp://jpbarrette:lddctr@localhost/"  UPLOAD_FILE_AS
#define RENAME_FILE_TO  "renamed-and-fine.txt"

int main(int, char **)
{
  // Get the file size of the local file (optional)
  // It seem to be the more standard C++ way to do it.
  // Open the file at the end of the file (ios::ate),
  // and return the read position (tellg)
  std::ifstream hd_src(LOCAL_FILE, std::ios::in | std::ios::ate | std::ios::binary);
  if(!hd_src) {
    std::cerr << "example13: Could not open the local file." << std::endl;
    return EXIT_FAILURE;
  }
  STREAMPOS hd_src_len = hd_src.tellg();
  
  // Rewinds the reading pos to the beginning.
  hd_src.seekg(0, std::ios::beg);

  // Now we build our request.
  try {
    // You will be sure that curlpp::initialize and
    // curlpp::terminate will be called.
    curlpp::cleanup myCleanup;

    // This is our request object.
    curlpp::ftp_easy request;

    /* build a list of commands to pass to libcurl */
    std::list< std::string > headerlist;
    headerlist.push_back("RNFR " UPLOAD_FILE_AS);
    headerlist.push_back("RNTO " RENAME_FILE_TO);

    /* enable uploading */
    request.upload();

    /* specify target */
    request.url(REMOTE_URL);

    /* pass in that last of FTP commands to run after the transfer */
    request.post_quote(headerlist);

    /* now specify which file to upload */
    curlpp::istream_trait my_trait(&hd_src);
    request.m_input_storage.trait(&my_trait, false);

    /* and give the size of the upload (optional) */
    request.infile_size(hd_src_len);

    /* Now run off and do what you've been told! */
    request.perform();
  }
  catch(curlpp::exception &e) {
    std::cerr << e.what() << std::endl;
    return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}
