#include <iostream>
#include <fstream>

#include <curlpp/curlpp.hpp>
#include <curlpp/ftp_easy.hpp>

/*
 * This example shows an FTP upload, with a rename of the file just after
 * a successful upload.
 *
 * Example based on libcurl's examples located in "doc/examples" source code.
 */

int main(int argc, char **argv)
{
  if(argc != 3) {
    std::cerr << "example14: Arguments missing." << std::endl;
    std::cerr << "example14: Usage: example14 remote_url remote_file" << std::endl;
    
    return EXIT_FAILURE;
  }

  char *remote_url = argv[1];
  char *remote_file = argv[2];

  std::ofstream my_header("example14_header.txt");
  curlpp::ostream_trait header_trait(&my_header);

  // Now we build our request.
  try {
    // You will be sure that curlpp::initialize and
    // curlpp::terminate will be called.
    curlpp::cleanup myCleanup;

    // This is our request object.
    curlpp::ftp_easy request;

    /* build a list of commands that contain the FTP SIZE */
    std::string sizeCommand("SIZE ");
    sizeCommand.append(remote_file);
    std::list< std::string > headerlist;
    headerlist.push_back(sizeCommand);

    /* specify target */
    request.url(remote_url);
    request.verbose();

    /* pass in that last of FTP commands to run after the transfer */
    request.post_quote(headerlist);

    request.m_header_storage.trait(&header_trait);

    /* Now run off and do what you've been told! */
    request.perform();
  }
  catch(curlpp::exception &e) {
    std::cerr << "example14: " << e.what() << std::endl;
    return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}
